import { MouseCursor } from "../../interfaces/frontend"
// import Bluetooth from "./Bluetooth"
// import FogImages from "./FogImages"
import Labeling from "./Labeling"
import Ships from "./Ships"
import BorderTiles from "@components/Gamefield/BorderTiles"
import EventBar from "@components/Gamefield/EventBar"
import HitElems from "@components/Gamefield/HitElems"
import Targets from "@components/Gamefield/Targets"
import { useDraw } from "@hooks/useDraw"
import { useDrawProps } from "@hooks/useDrawProps"
import { useGameProps } from "@hooks/useGameProps"
import useSocket from "@hooks/useSocket"
import { socket } from "@lib/socket"
import {
  initlialMouseCursor,
  overlapsWithAnyBorder,
  isAlreadyHit,
  targetList,
  shipProps,
} from "@lib/utils/helpers"
import { CSSProperties, useCallback } from "react"
import { useEffect, useState } from "react"

export const count = 12

function Gamefield() {
  const [mouseCursor, setMouseCursor] =
    useState<MouseCursor>(initlialMouseCursor)
  const {
    mode,
    hits,
    target,
    targetPreview,
    ships,
    addShip,
    payload,
    DispatchAction,
    setTarget,
    setTargetPreview,
    full,
  } = useGameProps()
  const { isConnected } = useSocket()

  useEffect(() => {
    if (payload?.game?.id || !isConnected) return
    socket.emit("update", full)
  }, [full, payload?.game?.id, isConnected])

  const settingTarget = useCallback(
    (isGameTile: boolean, x: number, y: number) => {
      if (payload?.game?.state === "running") {
        const list = targetList(targetPreview, mode)
        if (
          !isGameTile ||
          !list.filter(({ x, y }) => !isAlreadyHit(x, y, hits)).length
        )
          return
        if (target.show && target.x == x && target.y == y) {
          DispatchAction({
            action: "missile",
            ...target,
          })
          setTarget((t) => ({ ...t, show: false }))
        } else if (!overlapsWithAnyBorder(targetPreview, mode))
          setTarget({ show: true, x, y })
      } else if (payload?.game?.state === "starting") {
        addShip(shipProps(ships, mode, targetPreview))
      }
    },
    [
      DispatchAction,
      addShip,
      hits,
      mode,
      payload?.game?.state,
      setTarget,
      ships,
      target,
      targetPreview,
    ]
  )

  useEffect(() => {
    if (mode < 0) return
    const { x, y, show } = target
    const { shouldShow, ...position } = mouseCursor
    if (!shouldShow || overlapsWithAnyBorder(position, mode))
      setTargetPreview((e) => ({ ...e, show: false }))
    else {
      setTargetPreview({
        ...position,
        show: !show || x !== position.x || y !== position.y,
      })
    }
  }, [mode, mouseCursor, setTargetPreview, target])

  const { canvasRef, onMouseDown, clear } = useDraw()
  const { enable, color, shouldHide } = useDrawProps()

  return (
    <div id="gamefield">
      {/* <Bluetooth /> */}
      <div
        id="game-frame"
        style={{ "--i": count } as CSSProperties}
        onMouseLeave={() =>
          setMouseCursor((e) => ({ ...e, shouldShow: false }))
        }
      >
        {/* Bordes */}
        <BorderTiles props={{ settingTarget, setMouseCursor }} />

        {/* Collumn lettes and row numbers */}
        <Labeling count={count} />

        {/* Ships */}
        <Ships />
        <HitElems hits={hits} />

        {/* Fog images */}
        {/* <FogImages /> */}

        <Targets />

        <canvas
          style={
            {
              opacity: shouldHide ? 0 : 1,
              boxShadow: enable ? "inset 0 0 0 2px " + color : "none",
              pointerEvents: enable && !shouldHide ? "auto" : "none",
            } as CSSProperties
          }
          ref={canvasRef}
          onMouseDown={onMouseDown}
          width="648"
          height="648"
        />
      </div>
      <EventBar clear={clear} />
    </div>
  )
}

export default Gamefield
